require 'class'

-- FIELDS
local _StageList = {}
local _ControllerList = {}
local _BackStageList = {}
local _CurrentController = {}
local _OldController = {}
local _ShouldChangeScene = false
local NOT_FOUND = -1

-- CONSTRUCTOR
local M=class(function(self, stage, controllers)
	-- Store arguments
	_StageList = stage
	_ControllerList = controllers
	_CurrentController = nil
	_OldController = nil
end)

-- OPERATIONS
function M:createView(name, args)
	-- Cache the current controller as the old controller
	_OldController = _CurrentController
	-- Create the new controller
	local Controller = require(name)
	local aController = {}
	if args then
		aController = Controller(unpack(args))
	else
		aController = Controller()
	end
	table.insert(_ControllerList, aController)
	_CurrentController = aController
	-- Get the new view and add it to the back buffer
	local StageObjects = aController:getUI()
	if StageObjects then
		for Iter = 1, #StageObjects, 1 do
			table.insert(_BackStageList, StageObjects[Iter])
		end
	end
	-- Set the flag to change scenes
	_ShouldChangeScene = true
end

-- STAGE FRAMEWORK
function M:update(dt)
	if _ShouldChangeScene then
		-- Remove all the old elements from the stage
		local OldControllerObjects = {}
		if _OldController then
			OldControllerObjects = _OldController.getUI()
		end
		local Index = 0
		for Iter = 1, #OldControllerObjects, 1 do
			Index = doesTableContainElement(_StageList, OldControllerObjects[Iter])
			if Index ~= NOT_FOUND then
				table.remove(_StageList, Index)
			end
		end
		-- Remove the old controller from the controller list
		Index = doesTableContainElement(_ControllerList, _OldController)
		if Index ~= NOT_FOUND then
			table.remove(_ControllerList, Index)
		end
		-- Add new elements to the stage
		for Iter = 1, #_BackStageList, 1 do
			table.insert(_StageList, _BackStageList[Iter])
		end
		_BackStageList = {}
		_ShouldChangeScene = false
		_OldController = nil
		_CurrentController:update(dt)
	else
		_CurrentController:update(dt)
	end
end

-- LOGIC
-- Returns the index of an element in the given table if the element exists in the table, 
-- or -1 if it does not exist in the given table.
function doesTableContainElement(table, element)
	for Iter, value in ipairs(table) do
		if value == element then
			return Iter
		end
	end
	return -1
end

return M